package com.huimv.farm.damsubsidy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.entity.*;
import com.huimv.farm.damsubsidy.mapper.AreaByLevelMapper;
import com.huimv.farm.damsubsidy.service.IAreaByLevelService;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

// Author: Tinger-XXX
// Time: 2023/5/7 14:34
@Service
public class AreaByLevelServiceImpl implements IAreaByLevelService {
    @Autowired
    private AreaByLevelMapper dbMapper;

    @Override
    public Result listTypes() {
        List<AreaType> types = dbMapper.listTypes();
        return new Result(ResultCode.SUCCESS, types);
    }

    @Override
    public Result listLevels() {
        List<AreaLevel> levels = dbMapper.listLevels();
        return new Result(ResultCode.SUCCESS, levels);
    }

    @Override
    public Result getInfo() {
        List<AreaType> types = dbMapper.listTypes();
        List<AreaLevel> levels = dbMapper.listLevels();
        HashMap<String, Object> map = new HashMap<>();
        map.put("types", types);
        map.put("levels", levels);
        return new Result(ResultCode.SUCCESS, map);
    }

    @Override
    public Result listProvinces() {
        QueryWrapper<AreaAll> wrapper = new QueryWrapper<>();
        wrapper.eq("level", "1+");
        List<AreaAll> provinces = dbMapper.selectList(wrapper);
        return new Result(ResultCode.SUCCESS, provinces);
    }

    @Override
    public Result listChildrenOf(String currentId, String childLevel) {
        QueryWrapper<AreaAll> wrapper = new QueryWrapper<>();
        wrapper.likeRight("id", currentId)
                .eq("level", childLevel);
        List<AreaAll> children = dbMapper.selectList(wrapper);
        return new Result(ResultCode.SUCCESS, children);
    }

    @Override
    public Result treeOfTongWei() {
        AreaAll province = new AreaAll("62", "甘肃省", "1+", "2+", null),
                city = new AreaAll("6211", "定西市", "2+", "3+", null),
                county = new AreaAll("621121", "通渭县", "3+", "4+", null);
        QueryWrapper<AreaAll> townsWrapper = new QueryWrapper<>();
        townsWrapper.likeRight("id", "621121")
                .eq("level", "4+");
        List<AreaAll> towns = dbMapper.selectList(townsWrapper);
        JSONArray towns_json = new JSONArray();
        for (AreaAll town : towns) {
            QueryWrapper<AreaAll> villagesWrapper = new QueryWrapper<>();
            villagesWrapper.likeRight("id", town.getId())
                    .eq("level", town.getChild());
            List<AreaAll> villages = dbMapper.selectList(villagesWrapper);

            JSONObject town_json = JSONObject.fromObject(town);
            town_json.put("children", villages);
            towns_json.add(town_json);
        }

        JSONArray counties_json = new JSONArray(),
                cities_json = new JSONArray(),
                provinces_json = new JSONArray();
        JSONObject county_json = JSONObject.fromObject(county),
                city_json = JSONObject.fromObject(city),
                province_json = JSONObject.fromObject(province);
        county_json.put("children", towns_json);
        counties_json.add(county_json);
        city_json.put("children", counties_json);
        cities_json.add(city_json);
        province_json.put("children", cities_json);
        provinces_json.add(province_json);

        return new Result(ResultCode.SUCCESS, provinces_json);
    }

    @Override
    public Result insert(AreaAll one) {
        try {
            int res = dbMapper.insert(one);
            return new Result(ResultCode.SUCCESS, "已添加 <" + res + "> 条记录");
        } catch (Exception ignored) {
        }

        return new Result(10002, "添加失败", false);
    }

    @Override
    public Result insert(List<AreaAll> batch) {
        ArrayList<String> errId = new ArrayList<>();
        for (AreaAll one : batch) {
            try {
                dbMapper.insert(one);
            } catch (Exception ignored) {
                errId.add(one.getId());
            }
        }

        if (errId.size() == 0)
            return new Result(ResultCode.SUCCESS, "已添加 <" + batch.size() + "> 条记录");
        if (errId.size() != batch.size()) {
            int success = batch.size() - errId.size();
            return new Result(
                    ResultCode.SUCCESS,
                    "已添加 <" + success + "> 条记录, 以下id添加失败: " + errId
            );
        }

        return new Result(10002, "全部添加失败", false);
    }

    @Override
    public Result delete(String id) {
        try {
            int res = dbMapper.deleteById(id);
            return new Result(ResultCode.SUCCESS, "已删除 <" + res + "> 条记录");
        } catch (Exception ignored) {
        }

        return new Result(10002, "删除失败", false);
    }

    @Override
    public Result delete(List<String> ids) {
        try {
            int res = dbMapper.deleteBatchIds(ids);
            return new Result(ResultCode.SUCCESS, "已删除 <" + res + "> 条记录");
        } catch (Exception ignored) {
        }

        return new Result(10002, "删除失败", false);
    }

    @Override
    public Result update(AreaAll one) {
        try {
            int res = dbMapper.updateById(one);
            return new Result(ResultCode.SUCCESS, "已更新 <" + res + "> 条记录");
        } catch (Exception ignored) {
        }

        return new Result(10002, "更新失败", false);
    }

    @Override
    public Result update(List<AreaAll> batch) {
        ArrayList<String> errorId = new ArrayList<>();
        ArrayList<String> notExistId = new ArrayList<>();
        for (AreaAll one : batch) {
            try {
                int res = dbMapper.updateById(one);
                if (res == 0) notExistId.add(one.getId());
            } catch (Exception ignored) {
                errorId.add(one.getId());
            }
        }

        if (errorId.size() == 0 && notExistId.size() == 0)
            return new Result(ResultCode.SUCCESS, "已更新 <" + batch.size() + "> 条记录");
        if (errorId.size() != 0 && notExistId.size() != 0) {
            int success = batch.size() - errorId.size() - notExistId.size();
            if (success == 0)
                return new Result(
                        10002,
                        "更新失败，以下id不存在：" + notExistId + "，以下id更新失败：" + errorId,
                        false
                );
            return new Result(
                    ResultCode.SUCCESS,
                    "已更新 <" + success + "> 条记录，以下id不存在" + notExistId + "，以下id更新失败" + errorId
            );
        }
        if (notExistId.size() != 0) {
            int success = batch.size() - notExistId.size();
            return new Result(
                    ResultCode.SUCCESS,
                    "已更新 <" + success + "> 条记录, 以下id不存在: " + notExistId
            );
        }
        if (errorId.size() != batch.size()) {
            int success = batch.size() - errorId.size();
            return new Result(
                    ResultCode.SUCCESS,
                    "已更新 <" + success + "> 条记录, 以下id更新失败: " + errorId
            );
        }
        return new Result(10002, "全部更新失败", false);
    }
}
