package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.common.utils.UploadImage;
import com.huimv.farm.damsubsidy.entity.*;
import com.huimv.farm.damsubsidy.entity.vo.BillBuyInfoVo;
import com.huimv.farm.damsubsidy.entity.vo.BillLandingInspection;
import com.huimv.farm.damsubsidy.mapper.BaseAnimalMapper;
import com.huimv.farm.damsubsidy.mapper.BillBuyInfoMapper;
import com.huimv.farm.damsubsidy.mapper.BillLandingInspectionMapper;
import com.huimv.farm.damsubsidy.mapper.SysUserMapper;
import com.huimv.farm.damsubsidy.service.IBillBuyInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 交易信息 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillBuyInfoServiceImpl extends ServiceImpl<BillBuyInfoMapper, BillBuyInfo> implements IBillBuyInfoService {
    @Resource
    private BaseAnimalMapper baseAnimalMapper;
    @Resource
    private BillBuyInfoMapper billBuyInfoMapper;
    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private BillLandingInspectionMapper billLandingInspectionMapper;

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public Result addTransaction(HttpServletRequest httpServletRequest, MultipartFile image1, MultipartFile image2, MultipartFile image3, String inspectionNum, String seller, String buyer, String num, String eartags,String buyerPhone) throws IOException {
        BillBuyInfo billBuyInfo1 = billBuyInfoMapper.selectOne(new QueryWrapper<BillBuyInfo>().eq("create_user", TokenSign.getMemberIdByJwtToken(httpServletRequest)).eq("bill_buy_status",1));
        SysUser sysUser = sysUserMapper.selectOne(new QueryWrapper<SysUser>().eq("phone", buyerPhone));
        BillLandingInspection billLandingInspection = billLandingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", inspectionNum));

        Object savePoint = TransactionAspectSupport.currentTransactionStatus().createSavepoint();
        int sum = 0;
        int total = Integer.parseInt(num);

        if (ObjectUtil.isNotEmpty(sysUser)){
            if (!sysUser.getUserName().equals(buyer)){
                return new Result(10001,"买家手机号和姓名不匹配",false);
            }
        }
        if (eartags.contains(",")){
            eartags = eartags.replace("，",",");
            eartags = eartags.replace(", ",",");
            eartags = eartags.replace("， ",",");
            String[] eartag = eartags.split(",");
            for (String input : eartag) {
                if (input.contains("-")){
                    String start = input.substring(0, input.indexOf("-"));
                    if (start.length() != 15){
                        return new Result(10001,"耳标号"+input+"格式错误需输入15位数字",false);
                    }
                    String end = input.substring(input.indexOf("-") + 1);
                    if (end.length() != 3){
                        return new Result(10001,"耳标号"+input+"格式错误,-后需输入3位数字",false);
                    }
                    String substring1 = input.substring(start.length() - end.length(), start.length());
                    String substring = start.substring(0, start.length() - end.length());
                    if (Integer.parseInt(substring1) >Integer.parseInt(end) ){
                        return new Result(10001,"耳标号"+input+"格式错误,包含-号耳标号需递增",false);
                    }
                    List<String> eartagNos = new ArrayList<>();
                    for (int i = Integer.valueOf(substring1) ; i <=  Integer.valueOf(end); i++){
                        eartagNos.add(substring+i);
                    }
                    for (String number : eartagNos) {
                        BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", number).eq("inspection_num",inspectionNum));
                        if (ObjectUtil.isEmpty(baseAnimal)){
                            return new Result(10001,"耳标号"+number+"不存在",false);
                        }
                        if (baseAnimal.getIsDie() == 1){
                            return new Result(10001,"耳标号"+number+"畜种已死亡",false);
                        }
                        if (!baseAnimal.getBelongToUser().equals(billLandingInspection.getCreateUser())){
                            return new Result(10001,"耳标号"+number+"已交易,不属于此卖家!",false);
                        }
                        if (baseAnimal.getIsLoated() == 1){
                            return new Result(10001,"耳标号"+number+"已申请补贴,无法交易!",false);
                        }
                        if (ObjectUtil.isEmpty(sysUser)){
                            SysUser newUser = new SysUser();
                            newUser.setUserName(buyer);
                            newUser.setPhone(buyerPhone);
                            newUser.setUserType(3);
                            newUser.setAccoutType(1);
                            newUser.setCreateTime(DateTime.now());
                            newUser.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                            newUser.setIsRealName(0);
                            sysUserMapper.insert(newUser);
                            baseAnimal.setBelongToUser(newUser.getId());
                        }else {
                            baseAnimal.setBelongToUser(sysUser.getId());
                        }
                        baseAnimal.setBuyerName(buyer);
                        baseAnimal.setSellerName(seller);
                        baseAnimal.setAnimalStatus(2);
                        baseAnimal.setBuyTime(DateTime.now());
                        baseAnimalMapper.updateById(baseAnimal);
                        sum++;
                    }
                }else {
                    BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", input).eq("inspection_num",inspectionNum));
                    if (ObjectUtil.isEmpty(baseAnimal)){
                        return new Result(10001,"耳标号"+input+"不存在",false);
                    }
                    if (input.length() != 15){
                        return new Result(10001,"耳标号"+input+"格式错误",false);
                    }
                    Integer num2 = baseAnimalMapper.selectCount(new QueryWrapper<BaseAnimal>().eq("earatg_no", input));
                    if (num2 != 0){
                        return new Result(10001,"耳标号"+input+"重复",false);
                    }
                    if (baseAnimal.getIsDie() == 1){
                        return new Result(10001,"耳标号"+input+"畜种已死亡",false);
                    }
                    if (!baseAnimal.getBelongToUser().equals(billLandingInspection.getCreateUser())){
                        return new Result(10001,"耳标号"+input+"已交易,不属于此卖家!",false);
                    }
                    if (baseAnimal.getIsLoated() == 1){
                        return new Result(10001,"耳标号"+input+"已申请补贴,无法交易!",false);
                    }
                    if (ObjectUtil.isEmpty(sysUser)){
                        SysUser newUser = new SysUser();
                        newUser.setUserName(buyer);
                        newUser.setPhone(buyerPhone);
                        newUser.setUserType(3);
                        newUser.setAccoutType(1);
                        newUser.setCreateTime(DateTime.now());
                        newUser.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                        newUser.setIsRealName(0);
                        sysUserMapper.insert(newUser);
                        baseAnimal.setBelongToUser(newUser.getId());
                    }else {
                        baseAnimal.setBelongToUser(sysUser.getId());
                    }
                    baseAnimal.setBuyerName(buyer);
                    baseAnimal.setSellerName(seller);
                    baseAnimal.setAnimalStatus(2);
                    baseAnimal.setBuyTime(DateTime.now());
                    baseAnimalMapper.updateById(baseAnimal);
                    sum++;
                }
            }
        }else {
            if (eartags.contains("-")){
                String start = eartags.substring(0, eartags.indexOf("-"));
                if (start.length() != 15){
                    return new Result(10001,"耳标号"+eartags+"格式错误",false);
                }
                String end = eartags.substring(eartags.indexOf("-") + 1);
                if (end.length() != 3){
                    return new Result(10001,"耳标号"+eartags+"格式错误,-后需输入3位数字",false);
                }
                String substring1 = eartags.substring(start.length() - end.length(), start.length());
                String substring = start.substring(0, start.length() - end.length());
                System.out.println(substring);
                if (Integer.parseInt(substring1) >Integer.parseInt(end)){
                    return new Result(10001,"耳标号"+eartags+"格式错误,包含-号耳标号需递增",false);
                }
                List<String> eartagNos = new ArrayList<>();
                for (int i = Integer.valueOf(substring1) ; i <=  Integer.valueOf(end); i++){
                    eartagNos.add(substring+i);
                }
                for (String number : eartagNos) {
                    BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", number).eq("inspection_num",inspectionNum));
                    if (ObjectUtil.isEmpty(baseAnimal)){
                        return new Result(10001,"耳标号"+number+"不存在",false);
                    }
                    if (baseAnimal.getIsDie() == 1){
                        return new Result(10001,"耳标号"+number+"畜种已死亡",false);
                    }
                    if (!baseAnimal.getBelongToUser().equals(billLandingInspection.getCreateUser())){
                        return new Result(10001,"耳标号"+number+"已交易,不属于此卖家!",false);
                    }
                    if (baseAnimal.getIsLoated() == 1){
                        return new Result(10001,"耳标号"+number+"已申请补贴,无法交易!",false);
                    }
                    if (ObjectUtil.isEmpty(sysUser)){
                        SysUser newUser = new SysUser();
                        newUser.setUserName(buyer);
                        newUser.setUserType(3);
                        newUser.setPhone(buyerPhone);
                        newUser.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                        newUser.setAccoutType(1);
                        newUser.setCreateTime(DateTime.now());
                        newUser.setIsRealName(0);
                        sysUserMapper.insert(newUser);
                        baseAnimal.setBelongToUser(newUser.getId());
                    }else {
                        baseAnimal.setBelongToUser(sysUser.getId());
                    }
                    baseAnimal.setBuyerName(buyer);
                    baseAnimal.setSellerName(seller);
                    baseAnimal.setAnimalStatus(2);
                    baseAnimal.setBuyTime(DateTime.now());
                    baseAnimalMapper.updateById(baseAnimal);
                    sum++;
                }
            }else {
                BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", eartags).eq("inspection_num",inspectionNum));
                if (ObjectUtil.isEmpty(baseAnimal)){
                    return new Result(10001,"耳标号"+eartags+"不存在",false);
                }
                if (baseAnimal.getIsDie() == 1){
                    return new Result(10001,"耳标号"+eartags+"畜种已死亡",false);
                }
                if (!baseAnimal.getBelongToUser().equals(billLandingInspection.getCreateUser())){
                    return new Result(10001,"耳标号"+eartags+"已交易,不属于此卖家!",false);
                }
                if (baseAnimal.getIsLoated() == 1){
                    return new Result(10001,"耳标号"+eartags+"已申请补贴,无法交易!",false);
                }
                if (ObjectUtil.isEmpty(sysUser)){
                    SysUser newUser = new SysUser();
                    newUser.setUserName(buyer);
                    newUser.setUserType(3);
                    newUser.setAccoutType(1);
                    newUser.setPhone(buyerPhone);
                    newUser.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                    newUser.setCreateTime(DateTime.now());
                    newUser.setIsRealName(0);
                    sysUserMapper.insert(newUser);
                    baseAnimal.setBelongToUser(newUser.getId());
                }else {
                    baseAnimal.setBelongToUser(sysUser.getId());
                }
                baseAnimal.setBuyerName(buyer);
                baseAnimal.setSellerName(seller);
                baseAnimal.setAnimalStatus(2);
                baseAnimal.setBuyTime(DateTime.now());
                baseAnimalMapper.updateById(baseAnimal);
                sum++;
            }
        }

        if (total != sum){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new Result(10001,"实际耳标数量和填写数量不符!",false);
        }
        String content = UploadImage.getImageCom(image1);
        String content2 = UploadImage.getImageCom(image2);
        String content3 = UploadImage.getImageCom(image3);


        if (ObjectUtil.isEmpty(billBuyInfo1)){
            BillBuyInfo billBuyInfo = new BillBuyInfo();
            billBuyInfo.setBuyName(seller+"和"+buyer+"的交易信息");
            billBuyInfo.setInspectionNum(inspectionNum);
            billBuyInfo.setBuyerName(buyer);
            billBuyInfo.setSellerName(seller);
            billBuyInfo.setBuyerPhone(buyerPhone);
            billBuyInfo.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
            billBuyInfo.setCreateTime(DateTime.now());
            billBuyInfo.setRealNumUp(num);
            billBuyInfo.setBuyPicUrl(content);
            billBuyInfo.setSellerId(billLandingInspection.getCreateUser());
            billBuyInfo.setPayPicUrl(content2);
            billBuyInfo.setBuyPayGoodsPicUrl(content3);
            billBuyInfo.setRealEartag(eartags);
            billBuyInfo.setBillBuyStatus(2);
            this.save(billBuyInfo);
        }else {
            billBuyInfo1.setBuyName(seller+"和"+buyer+"的交易信息");
            billBuyInfo1.setInspectionNum(inspectionNum);
            billBuyInfo1.setBuyerName(buyer);
            billBuyInfo1.setSellerName(seller);
            billBuyInfo1.setBuyerPhone(buyerPhone);
            billBuyInfo1.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
            billBuyInfo1.setSellerId(billLandingInspection.getCreateUser());
            billBuyInfo1.setCreateTime(DateTime.now());
            billBuyInfo1.setRealNumUp(num);
            billBuyInfo1.setBuyPicUrl(content);
            billBuyInfo1.setPayPicUrl(content2);
            billBuyInfo1.setBuyPayGoodsPicUrl(content3);
            billBuyInfo1.setRealEartag(eartags);
            billBuyInfo1.setBillBuyStatus(2);
            this.updateById(billBuyInfo1);
        }
        if(ObjectUtil.isEmpty(sysUser)){
            return new Result(10003,"交易成功买家用户信息不存在,已新建账号",true);
        }
        return Result.SUCCESS();
    }

    @Override
    @Transactional
    public Result saveTransaction(HttpServletRequest httpServletRequest,Map<String, String> paramsMap) {
        String inspectionNum = paramsMap.get("inspectionNum");
        String seller = paramsMap.get("seller");
        String buyer = paramsMap.get("buyer");
        String num = paramsMap.get("num");
        String eartags = paramsMap.get("eartags");
        String buyerPhone = paramsMap.get("buyerPhone");
//        String imgname = "成功";
//        String originalFilename = image1.getOriginalFilename();
//        String filenameExtension = StringUtils.getFilenameExtension(originalFilename);
//        String path = DateUtil.format(new Date(),"yyyy-MM");
//        try {
//            InputStream inputStream = image1.getInputStream();
//            BufferedInputStream in = new BufferedInputStream(inputStream);
//            ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
//
//            byte[] temp = new byte[1024];
//            int size = 0;
//            while ((size = in.read(temp)) != -1) {
//                out.write(temp, 0, size);
//            }
//            in.close();
//            byte[] content = out.toByteArray();
//            imgname = UUID.randomUUID()+"."+filenameExtension;
//            UploadImage.sshSftp(content,path, imgname);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        String content = "https://img.ifarmcloud.com/images/"+path +"/"+imgname;
//
//        String imgname2 = "成功";
//        String originalFilename2 = image2.getOriginalFilename();
//        String filenameExtension2 = StringUtils.getFilenameExtension(originalFilename2);
//        String path2 = DateUtil.format(new Date(),"yyyy-MM");
//        try {
//            InputStream inputStream = image2.getInputStream();
//            BufferedInputStream in = new BufferedInputStream(inputStream);
//            ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
//
//            byte[] temp = new byte[1024];
//            int size = 0;
//            while ((size = in.read(temp)) != -1) {
//                out.write(temp, 0, size);
//            }
//            in.close();
//            byte[] content2 = out.toByteArray();
//            imgname2 = UUID.randomUUID()+"."+filenameExtension2;
//            UploadImage.sshSftp(content2,path2, imgname2);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        String content2 = "https://img.ifarmcloud.com/images/"+path2 +"/"+imgname2;
//
//        String imgname3 = "成功";
//        String originalFilename3 = image3.getOriginalFilename();
//        String filenameExtension3 = StringUtils.getFilenameExtension(originalFilename3);
//        String path3 = DateUtil.format(new Date(),"yyyy-MM");
//        try {
//            InputStream inputStream = image3.getInputStream();
//            BufferedInputStream in = new BufferedInputStream(inputStream);
//            ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
//
//            byte[] temp = new byte[1024];
//            int size = 0;
//            while ((size = in.read(temp)) != -1) {
//                out.write(temp, 0, size);
//            }
//            in.close();
//            byte[] content3 = out.toByteArray();
//            imgname3 = UUID.randomUUID()+"."+filenameExtension3;
//            UploadImage.sshSftp(content3,path3, imgname3);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        String content3 = "https://img.ifarmcloud.com/images/"+path3 +"/"+imgname3;

        BillBuyInfo billBuyInfo1 = billBuyInfoMapper.selectOne(new QueryWrapper<BillBuyInfo>().eq("create_user", TokenSign.getUserName(httpServletRequest)).eq("bill_buy_status",1));
        if (ObjectUtil.isEmpty(billBuyInfo1)){
            BillBuyInfo billBuyInfo = new BillBuyInfo();
            billBuyInfo.setBuyName(seller+"和"+buyer+"的交易信息");
            billBuyInfo.setInspectionNum(inspectionNum);
            billBuyInfo.setBuyerName(buyer);
            billBuyInfo1.setBuyerPhone(buyerPhone);
            billBuyInfo.setSellerName(seller);
            billBuyInfo.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
            billBuyInfo.setCreateTime(DateTime.now());
            billBuyInfo.setRealNumUp(num);
//            billBuyInfo.setBuyPicUrl(content);
//            billBuyInfo.setPayPicUrl(content2);
//            billBuyInfo.setBuyPayGoodsPicUrl(content3);
            billBuyInfo.setRealEartag(eartags);
            billBuyInfo.setBillBuyStatus(1);
            this.save(billBuyInfo);
        }else {
            billBuyInfo1.setBuyName(seller+"和"+buyer+"的交易信息");
            billBuyInfo1.setInspectionNum(inspectionNum);
            billBuyInfo1.setBuyerName(buyer);
            billBuyInfo1.setBuyerPhone(buyerPhone);
            billBuyInfo1.setSellerName(seller);
            billBuyInfo1.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
            billBuyInfo1.setCreateTime(DateTime.now());
            billBuyInfo1.setRealNumUp(num);
//            billBuyInfo1.setBuyPicUrl(content);
//            billBuyInfo1.setPayPicUrl(content2);
//            billBuyInfo1.setBuyPayGoodsPicUrl(content3);
            billBuyInfo1.setRealEartag(eartags);
            billBuyInfo1.setBillBuyStatus(1);
            this.updateById(billBuyInfo1);
        }
        return Result.SUCCESS();
    }

    @Override
    public Result getLast(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        BillBuyInfoVo billBuyInfo = billBuyInfoMapper.getLast(TokenSign.getMemberIdByJwtToken(httpServletRequest));
        if (ObjectUtil.isEmpty(billBuyInfo)){
            return new Result(ResultCode.SUCCESS,null);
        }else {
            return new Result(ResultCode.SUCCESS,billBuyInfo);
        }
    }

    @Override
    public Result listDetail(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        BillBuyInfoVo billBuyInfo= billBuyInfoMapper.listDetail(id);
        return new Result(ResultCode.SUCCESS,billBuyInfo);
    }
}
