package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.common.utils.UploadImage;
import com.huimv.farm.damsubsidy.entity.BaseAnimal;
import com.huimv.farm.damsubsidy.entity.BillIsolated;
import com.huimv.farm.damsubsidy.entity.vo.BillLandingInspection;
import com.huimv.farm.damsubsidy.mapper.BaseAnimalMapper;
import com.huimv.farm.damsubsidy.mapper.BillIsolatedMapper;
import com.huimv.farm.damsubsidy.mapper.BillLandingInspectionMapper;
import com.huimv.farm.damsubsidy.service.IBillIsolatedService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;

/**
 * <p>
 * 落地检 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillIsolatedServiceImpl extends ServiceImpl<BillIsolatedMapper, BillIsolated> implements IBillIsolatedService {

    @Resource
    private BillLandingInspectionMapper landingInspectionMapper;

    @Resource
    private BaseAnimalMapper baseAnimalMapper;

    @Override
    @Transactional
    public synchronized Result addIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String inspectionNum = paramsMap.get("inspectionNum");
        String isolatedPosition = paramsMap.get("isolatedPosition");
        String strIsolated = paramsMap.get("strIsolated");
        String dayIsolated = paramsMap.get("dayIsolated");
        int dayIsolate = Integer.parseInt(dayIsolated);
        if (com.baomidou.mybatisplus.core.toolkit.StringUtils.isBlank(inspectionNum) || inspectionNum.length()!=10){
            return new Result(10001,"检疫证号不符合规范！",false);
        }
        if (dayIsolate > 20 || dayIsolate <15){
            return new Result(10001,"隔离天数请在15-20天！",false);
        }
        Integer integer = this.count(new QueryWrapper<BillIsolated>().eq("inspection_num", inspectionNum));
        if (integer != 0){
            return new Result(10001,"检疫证号重复！",false);
        }
        BillIsolated billIsolated = new BillIsolated();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date startTime = dateFormat.parse(strIsolated);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);
        calendar.add(Calendar.DATE, dayIsolate);
        Date endTime = calendar.getTime();
        String userName = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", inspectionNum)).getCargoOwner();
        billIsolated.setInspectionName(userName+"的隔离信息");
        billIsolated.setInspectionNum(inspectionNum);
        billIsolated.setIsolatedPosition(isolatedPosition);
        billIsolated.setStrIsolated(startTime);
        billIsolated.setDayIsolated(Integer.parseInt(dayIsolated));
        billIsolated.setEndIsolated(endTime);
        billIsolated.setCreateTime(DateTime.now());
        billIsolated.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));


        BillLandingInspection billLandingInspection = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", inspectionNum).ne("inspection_sch", 5));
        String eartags = billLandingInspection.getEartags();
        eartags = eartags.replace("，",",");
        eartags = eartags.replace(", ",",");
        eartags = eartags.replace("， ",",");
        if (eartags.contains(",")){
            String[] eartag = eartags.split(",");
            for (String input : eartag) {
                if (input.contains("-")){
                    String start = input.substring(0, input.indexOf("-"));
                    if (start.length() != 15){
                        return new Result(10001,"耳标号"+input+"格式错误需输入15位数字",false);
                    }
                    String end = input.substring(input.indexOf("-") + 1);
                    if (end.length() != 3){
                        return new Result(10001,"耳标号"+input+"格式错误,-后需输入3位数字",false);
                    }
                    String substring1 = input.substring(start.length() - end.length(), start.length());
                    String substring = start.substring(0, start.length() - end.length());
                    System.out.println(substring);
                    List<String> eartagNos = new ArrayList<>();
                    if (Integer.parseInt(substring1) >Integer.parseInt(end) ){
                        return new Result(10001,"耳标号"+input+"格式错误,包含-号耳标号需递增",false);
                    }
                    for (int i = Integer.parseInt(substring1); i <=  Integer.parseInt(end); i++){
                       String earTag  = getEndEartag(substring,i);
                        eartagNos.add(earTag);
                    }
                    for (String eartagNo : eartagNos) {
                        Integer num = baseAnimalMapper.selectCount(new QueryWrapper<BaseAnimal>().eq("earatg_no", eartagNo));
                        if (num != 0){
                            return new Result(10001,"耳标号"+eartagNo+"重复",false);
                        }
                        BaseAnimal baseAnimal = new BaseAnimal();
                        baseAnimal.setEaratgNo(eartagNo);
                        baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                        baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                        baseAnimal.setBelongToUser(billLandingInspection.getCreateUser());
                        baseAnimal.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                        baseAnimal.setCreateTime(new Date());
                        baseAnimalMapper.insert(baseAnimal);
                    }
                }else {
                    if (input.length() != 15){
                        return new Result(10001,"耳标号"+input+"格式错误需输入15位数字",false);
                    }
                    Integer num = baseAnimalMapper.selectCount(new QueryWrapper<BaseAnimal>().eq("earatg_no", input));
                    if (num != 0){
                        return new Result(10001,"耳标号"+input+"重复",false);
                    }
                    BaseAnimal baseAnimal = new BaseAnimal();
                    baseAnimal.setEaratgNo(input);
                    baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                    baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                    baseAnimal.setBelongToUser(billLandingInspection.getCreateUser());
                    baseAnimal.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                    baseAnimal.setCreateTime(new Date());
                    baseAnimalMapper.insert(baseAnimal);
                }
            }
        }else {
            if (eartags.contains("-")){
                String start = eartags.substring(0, eartags.indexOf("-"));
                if (start.length() != 15){
                    return new Result(10001,"耳标号"+eartags+"格式错误需输入15位数字",false);
                }
                String end = eartags.substring(eartags.indexOf("-") + 1);
                if (end.length() != 3){
                    return new Result(10001,"耳标号"+eartags+"格式错误,-后需输入3位数字",false);
                }
                String substring1 = eartags.substring(start.length() - end.length(), start.length());
                String substring = start.substring(0, start.length() - end.length());
                System.out.println(substring);
                List<String> eartagNos = new ArrayList<>();
                if (Integer.parseInt(substring1) >Integer.parseInt(end)){
                    return new Result(10001,"耳标号"+eartags+"格式错误,包含-号耳标号需递增",false);
                }
                for (int i = Integer.valueOf(substring1) ; i <=  Integer.valueOf(end); i++){
                    String earTag  = getEndEartag(substring,i);
                    eartagNos.add(earTag);
                }
                for (String number : eartagNos) {
                    Integer num = baseAnimalMapper.selectCount(new QueryWrapper<BaseAnimal>().eq("earatg_no", number));
                    if (num != 0){
                        return new Result(10001,"耳标号"+number+"重复",false);
                    }
                    BaseAnimal baseAnimal = new BaseAnimal();
                    baseAnimal.setEaratgNo(number);
                    baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                    baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                    baseAnimal.setBelongToUser(billLandingInspection.getCreateUser());
                    baseAnimal.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                    baseAnimal.setCreateTime(new Date());
                    baseAnimalMapper.insert(baseAnimal);
                }
            }else {
                if (eartags.length() != 15){
                    return new Result(10001,"耳标号"+eartags+"格式错误需输入15位数字",false);
                }
                Integer num = baseAnimalMapper.selectCount(new QueryWrapper<BaseAnimal>().eq("earatg_no", eartags));
                if (num != 0){
                    return new Result(10001,"耳标号"+eartags+"重复",false);
                }
                BaseAnimal baseAnimal = new BaseAnimal();
                baseAnimal.setEaratgNo(eartags);
                baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                baseAnimal.setBelongToUser(billLandingInspection.getCreateUser());
                baseAnimal.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
                baseAnimal.setCreateTime(new Date());
                baseAnimalMapper.insert(baseAnimal);
            }
        }
        billLandingInspection.setInspectionSch(3);
        billLandingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setReviewedTime(DateTime.now());
        landingInspectionMapper.updateById(billLandingInspection);
        this.save(billIsolated);
        return Result.SUCCESS();

    }

    private String getEndEartag(String substring, int i) {
        StringBuilder endEartag = new StringBuilder(i + "");
        int lenth = String.valueOf(i).length();
        if (lenth ==3) {
            return substring+i;
        }else {
            for (int i1 = 0; i1 < 2; i1++) {
                endEartag.insert(0, "0");
                if (endEartag.length() ==3){
                    return substring+endEartag;
                }
            }
        }

        return substring+i;
    }



    @Override
    public Result listIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String type = paramsMap.get("type");
        Page<BillIsolated> page = new Page(Integer.parseInt(pageNum),Integer.parseInt(pageSize));
        if (type.equals("1")){
            return new Result(ResultCode.SUCCESS,this.page(page,new QueryWrapper<BillIsolated>().ge("str_isolated",LocalDate.now()).le("end_isolated",LocalDate.now())));
        }else {
            return new Result(ResultCode.SUCCESS,this.page(page,new QueryWrapper<BillIsolated>().lt("end_isolated",LocalDate.now())));
        }
    }

    @Override
    public Result delayedIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String day = paramsMap.get("day");
        if (day.contains(".")){
            return new Result(10001,"请输入整数",false);
        }
        BillIsolated billIsolated = this.getById(id);
        billIsolated.setDayIsolated(billIsolated.getDayIsolated()+Integer.parseInt(day));
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(billIsolated.getEndIsolated());
        calendar.add(Calendar.DATE, Integer.valueOf(day));
        Date endTime = calendar.getTime();
        billIsolated.setUpdateUser(TokenSign.getUserName(httpServletRequest));
        billIsolated.setUpdateTime(DateTime.now());
        billIsolated.setEndIsolated(endTime);
        this.updateById(billIsolated);
        return Result.SUCCESS();
    }

    @Override
    @Transactional
    public Result cancelIsolation(HttpServletRequest httpServletRequest, MultipartFile image, String id, String isDead, String deaths) {
        String imgname = "成功";
        String originalFilename = image.getOriginalFilename();
        String filenameExtension = StringUtils.getFilenameExtension(originalFilename);
        String path = DateUtil.format(new Date(),"yyyy-MM");
        try {
            InputStream inputStream = image.getInputStream();
            BufferedInputStream in = new BufferedInputStream(inputStream);
            ByteArrayOutputStream out = new ByteArrayOutputStream(1024);

            byte[] temp = new byte[1024];
            int size = 0;
            while ((size = in.read(temp)) != -1) {
                out.write(temp, 0, size);
            }
            in.close();
            byte[] content = out.toByteArray();
            imgname = UUID.randomUUID()+"."+filenameExtension;
            UploadImage.sshSftp(content,path, imgname);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String content = "https://img.ifarmcloud.com/images/"+path +"/"+imgname;
        BillIsolated isolation = this.getById(id);
        isolation.setIsolatedPicUrl(content);
        isolation.setIsolatedSch(2);

        deaths = deaths.replace("，",",");
        deaths = deaths.replace(", ",",");
        deaths = deaths.replace("，  ",",");
        if ("1".equals(isDead)){
            isolation.setIsDead(1);
            isolation.setDeaths(deaths);
            if (deaths.contains(",")){
                String[] eartag = deaths.split(",");
                for (String s : eartag) {
                    if (!s.contains("-")){
                        BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", s));
                        if (ObjectUtil.isEmpty(baseAnimal)){
                            return new Result(10001,"耳标号"+s+"不存在",false);
                        }
                        baseAnimal.setIsDie(1);
                        baseAnimalMapper.updateById(baseAnimal);
                    }else {
                        String start = s.substring(0, s.indexOf("-"));
                        if (start.length() != 15){
                            return new Result(10001,"耳标号"+s+"格式错误需输入15位数字",false);
                        }
                        String end = s.substring(s.indexOf("-") + 1);
                        if (end.length() != 3){
                            return new Result(10001,"耳标号"+s+"格式错误,-后需输入3位数字",false);
                        }
                        String substring1 = s.substring(start.length() - end.length(), start.length());
                        String substring = start.substring(0, start.length() - end.length());
                        System.out.println(substring);
                        if (Integer.parseInt(substring1) >Integer.parseInt(end) ){
                            return new Result(10001,"耳标号"+s+"格式错误,包含-号耳标号需递增",false);
                        }
                        List<String> eartagNos = new ArrayList<>();
                        for (int i = Integer.valueOf(substring1) ; i <=  Integer.valueOf(end); i++){
                            String earTag  = getEndEartag(substring,i);
                            eartagNos.add(earTag);
                        }
                        for (String number : eartagNos) {
                            BaseAnimal animal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", number));
                            if (ObjectUtil.isEmpty(animal)){
                                return new Result(10001,"耳标号"+number+"不存在",false);
                            }
                            animal.setIsDie(1);
                            baseAnimalMapper.updateById(animal);
                        }
                    }
                }
            }else {
                if (!deaths.contains("-")){
                    BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", deaths));
                    if (ObjectUtil.isEmpty(baseAnimal)){
                        return new Result(10001,"耳标号"+deaths+"不存在",false);
                    }
                    baseAnimal.setIsDie(1);
                    baseAnimalMapper.updateById(baseAnimal);
                }else {
                    String start = deaths.substring(0, deaths.indexOf("-"));
                    if (start.length() != 15){
                        return new Result(10001,"耳标号"+deaths+"格式错误需输入15位数字",false);
                    }
                    String end = deaths.substring(deaths.indexOf("-") + 1);
                    if (end.length() != 3){
                        return new Result(10001,"耳标号"+deaths+"格式错误,-后需输入3位数字",false);
                    }
                    String substring1 = deaths.substring(start.length() - end.length(), start.length());
                    String substring = start.substring(0, start.length() - end.length());
                    if (Integer.parseInt(substring1) >Integer.parseInt(end)){
                        return new Result(10001,"耳标号"+deaths+"格式错误,包含-号耳标号需递增",false);
                    }
                    List<String> eartagNos = new ArrayList<>();
                    for (int i = Integer.valueOf(substring1) ; i <=  Integer.valueOf(end); i++){
                        String earTag  = getEndEartag(substring,i);
                        eartagNos.add(earTag);
                    }
                    for (String number : eartagNos) {
                        // TODO
                        BaseAnimal animal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", number));
                        if (ObjectUtil.isEmpty(animal)){
                            return new Result(10001,"耳标号"+number+"不存在",false);
                        }
                        animal.setIsDie(1);
                        baseAnimalMapper.updateById(animal);
                    }
                }
            }
        }else {
            isolation.setIsDead(0);
        }
        List<BaseAnimal> baseAnimals = baseAnimalMapper.selectList(new QueryWrapper<BaseAnimal>().eq("inspection_num", isolation.getInspectionNum()));
        for (BaseAnimal baseAnimal : baseAnimals) {
            baseAnimal.setAnimalStatus(1);
            baseAnimalMapper.updateById(baseAnimal);
        }
        BillLandingInspection billLandingInspection = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", isolation.getInspectionNum()));
        billLandingInspection.setReviewedTime(DateTime.now());
        billLandingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setInspectionSch(4);
        landingInspectionMapper.updateById(billLandingInspection);
        isolation.setUpdateTime(DateTime.now());
        isolation.setUpdateUser(TokenSign.getUserName(httpServletRequest));
        this.updateById(isolation);
        return Result.SUCCESS();
    }

}
